from setuptools import setup, find_packages

setup(name='edecs',
      version='0.1.11',
      description='Event-driven Entity-Component-System engine',
      packages=find_packages(),
      author_email='dovakin121212@gmail.com',
      zip_safe=False)
